/* Instructables
 *  Project Eagle I
 *  Wireless_Sentry_Defence_
 *  
 *  Authors: Shafae Ali, Oscar Jones
 *  
 *  NANO File
 *  The project utilises an Arduino UNO (which controls the turret) and an Arduino Nano (which controls the joystick).
 *  Both Arduinos are wirelessly communicating with each other on a 2.4GHz wireless network, resulting in two seperate .ino files.
 *  One is for inputting commands (the joystick/Nano), the other is for outputting the data sent to it (the turret/UNO).
 *  The following code is for the turret and is used on the Arduino NANO.
 *  One key note for the script is that care has been taken to avoid any lines of code that cause interuptions or delays in other commands taking place.
 *  The script has been written with the intention to focus on rapid response, given than it is operating wirelessly.
 *  The implementation reflects this as in actual use, there is negligible delay between turret outcome and joystick interaction.
 */


//the SPI library is used for communicating with the NRF24L01 module. 
//It uses the SCK, MISO and MOSI pins for communicating quickly over short distances, e.g. wireless module, RTC module
#include <SPI.h>
//nRF24L01 and RF24 are libraries for using the wireless module
#include <nRF24L01.h>
#include <RF24.h>

//this is to define the wireless module pins seperately to the rest of the code. This makes redfining them simple as it is core to our data transmission
//the CE_PIN is used for data transmission and is the primary pin for the wireless module
#define CE_PIN 9
//the CSN pin is an active low pin for the SPI bus
#define CSN_PIN 10

//an address for communicating over is established as a read-only variable using const
//a random selection of letters was chosen as a unqiue address
const byte address[5] = {'R', 'x', 'A', 'A', 'A'};

//use of RF24 in which the two non-standard pins are defined for radio communication
//SCK, MOSI and MISO are pre-defined as pin 13,12 and 11, whereas CE and CSN are chosen
RF24 radio(CE_PIN, CSN_PIN);

//a single array that holds 4 values which will be sent to the Arduino UNO
//This is more efficient and robust than trying to send 4 seperate values
int message[4];

//Groupings of the component pin and associated data being used with it have been made

//defining the digital pin for the blue wireless connection status LED
//defining the digital pin for the red arming state of the motors LED
int connection_led_pin = 8;
int arming_led_pin = 7;

//Potentiometer for defining the tilt angle on the servo
//associated value from the potentiometer
int angle_pot = A0;
int angle_val;

//Potentiometer for defining the swivel on the servo
//associated value from the potentiometer
int rotation_pot = A1;
int rotation_val;

//setting the arming pin and grouping it with corresponding code which processes the values given to the ESC
const byte arming_pin = 3;
//this particular button has a tendency to stick during use, therefore button states have been defined to debounce them later
int escButtonState = 0;
//the ESC needs values consistently written to it for it to function as intended, therefore an array has been generated of the two speeds values that will be iterated between when the button is pressed
int escSpeeds[2] = {1500, 1575};
//this is a button counter that will continously increment from 0
int buttonPushCounter = 0;
//used for debouncing
int lastButtonState = 0;
//a single value will be sent in the message array to the UNO
int escSpeed;

//Setting the firing pin and its state
//The firing pin is LOW when pressed and HIGH when not pressed
const byte firing_pin = 5;
int firing_state;


/////////////////////////////////////////////////////////

void setup() {
  
  //this was initally used in setup during troubleshooting and iteratively devloping code whilst interfacing to the hardware
  //it has been commented out for now, but has been kept for quick development requiring reading of values, etc.
  //Serial.begin(9600);

  //this ties the input to high as the switch is wired in this way
  //The joystick casing made it desirable to wire it as such due to tieing buttons to ground and having space for the data inputs too
  pinMode(arming_pin, INPUT_PULLUP);
  //Defining both sets of LEDs as outputs
  pinMode(connection_led_pin, OUTPUT);
  pinMode(arming_led_pin, OUTPUT);
  
  //this starts radio transmission and utilises the RF24 module
  radio.begin();
  //the lowest data transfer rate provides greater connection range
  //since a single array is being sent, this rate is suitable as the more important factor is the wireless range
  radio.setDataRate( RF24_250KBPS );
  //the radio module will transmit the message up to 5 times with 3s between messages until it has been received by the UNO
  radio.setRetries(3, 5);
  //the nRF24L01 has 6 pipes that it can transmit or receive data to at a time
  //this command opens the first data pipe for writing at the address we defined earlier
  radio.openWritingPipe(address);
  //when the joystick is powered on, the LED will display it is transmitting messages
  digitalWrite(connection_led_pin, HIGH);
  //to ensure there is no interference in the data being sent, the Nano shuts off its ability to receive messages with this command and is only acting as a transmitter
  radio.stopListening();
}

/////////////////////////////////////////////////////////

void loop() {
  
  ////////////////TURRET TILT POT READING//////////////////
  //reading the potentiometer values from the analog pin, this is between 0 and 1023
  angle_val = analogRead(angle_pot);
  //since the potentiometer is mechanically fixed in the joystick, it only travels between 390 and 600
  //in order to write a suitable value to the servo, the potentiometer value is mapped accordingly
  //this keeps all the functionality of the potentiometer code within the Nano making reprogramming and adapting of the input controls later easier
  angle_val = map(angle_val, 390, 600, 105, 160);
  
  ////////////////TURRET SWIVEL POT READING//////////////////
  //Identical process to tilt pot
  //Adjusting this value for suitability on a continous servo is done on the UNO
  rotation_val = analogRead(rotation_pot);
  rotation_val = map(rotation_val, 390, 600, 0, 250);
  
  ////////////////ARMING BUTTON//////////////////
  //Assigning button state to the reading from the connected pin
  escButtonState = digitalRead(arming_pin);

  //last button state is defined as 0, therefore if both are not equal to each other initiate if statement
  if (escButtonState != lastButtonState)
  {
    //when the button is low add 1 to the counter, but only if the above if statement holds true
    //the reason for this is so that the button is not continously read and it is only the change in button going from high to low to high, does the counter increment
    //this is important for ESC control as it ensures that there is no conflict between continously writing the button and continously sending the ESC value
    if (escButtonState == LOW)
    {
      buttonPushCounter++;
    }
  }
  //resets the button state
  lastButtonState = escButtonState;
  //setting the array value to a single value that will be sent in the wireless message
  //the escSpeeds array is iterated between the first value and seconds value by using the modulus function, it will take the remainder of the button push counter value
  //implementing the code in this way makes the counter function cleanly inserted in a single line
  //additonal speeds can be implemented, changing the arming button to a speed cycling button, however for the purpose of two on off motor states, this is suitable
  escSpeed = escSpeeds[buttonPushCounter % 2];
  
  ////////////////ARMING LED//////////////////
  //A very simple segment of the code where depending on the value set by ESC speed, the arming LED will either be on or off
  //The arming LED is important as it acts as a safety indicator to everyone that the high speed motors are operational
  if (escSpeed == 1500){
  digitalWrite(arming_led_pin, LOW);
  }
  
  if (escSpeed >= 1550){
  digitalWrite(arming_led_pin, HIGH);
  }
  
  ////////////////FIRING BUTTON//////////////////
  //Simple reading of the state of the firing button
  //The implementaion of this code to an output is done on the UNO where all the turret outputs are located
  firing_state = digitalRead(firing_pin);
  
  ////////////////RADIO TRANSMISSION//////////////////
  //This is setting each of the values that have been read using the Nano into a single array defined as 'message'
  //Variable names will remain synonymous across microcontrollers to ensure clear understanding of code
  //each variable name is self-identifying to the turret
  message[0] = angle_val;
  message[1] = rotation_val;
  message[2] = escSpeed;
  message[3] = firing_state;

  //'sizeof' is a command for writing the message to the address, the second argument confirms the number of bytes occupied in the array and how many should be received
  //using '&' in this instance will pass the address of the variable where the data is stored
  radio.write(&message, sizeof(message));

}
